function run()
% Author: Asad Mohammadi
% Email : asadmoh1985@gmail.com
% -----------
% References:
% -----------
%  Mohammadi, A., Omidvar, M. and Li, X. (2012), "Reference Point Based Multi-objective Optimization Through Decomposition", 
%  in Proceedings of Congress of Evolutionary Computation (CEC 2012), IEEE,  p.1150 - 1157.
% --------
% License:
% --------
% This program is to be used under the terms of the GNU General Public License 
% (http://www.gnu.org/copyleft/gpl.html).
% Author: Asad Mohammadi
% Email : asadmoh1985@gmail.com
% Copyright notice: (c) 2014 Asad Mohammadi
% In this function, you can specify the reference points, size of preferred
% region, method of decomposition (ws,te), population size, number of
% iterations and test-problem
    nruns = 1;
    for r = 1:nruns
        clear all;
        close all;

        dim = 11;
        %test = 'kno1';
        %test = 'zdt1';
        %test = 'zdt2';
        %test = 'zdt3';
        %test = 'zdt4';
        %test = 'zdt5';
        %test = 'zdt6';
        %test = 'dtlz1_2d';
        test = 'dtlz1_3d';
        %test = 'dtlz2';

        mop = testmop(test, dim);
        figure(1);
        plotmop(test, mop);
        drawnow;
        hold on;    
        subproblems = moead(mop,'popsize', 231, 'niche', 20, 'iteration', 50, 'method', 'te', 'plotting', 0, 'refresh', 0, 'save', 0);
        refs =[[0.5;0.5;0.5]];
        vars.popsize = 100;
        vars.niche = vars.popsize;
        vars.iteration = 300;
        vars.method = 'te';
        vars.radius = 0.08;
        vars.density = vars.popsize; 
        vars.test = test;
        vars.plotting = 0;
        vars.refresh = 0;
        pareto=[subproblems.curpoint];
        pp=[pareto.objective];
        numref = size(refs, 2);
        vars.base_weight = [];
        vars.ref=[];
        figure(2);
        plotmop(test, mop); drawnow;
        for rp=1:numref
            temprefs  = refs(:,rp) * ones(1, length(subproblems)); 
            dist = sqrt(sum((pp-temprefs).^2));
            [Y I] = min(dist);
            temp = subproblems(I).weight;
            vars.base_weight = temp;
            vars.ref=refs(:,rp);
            pareto_ref=moead_up(mop, vars);
            pp_ref=[pareto_ref.objective];
            pp_ref = [pp_ref refs(:,rp)];
            figure(2);
          
            hold on;
            if(mop.od == 2)
                scatter(pp_ref(1,:), pp_ref(2,:));
                x = refs(1, rp);
                y = refs(2, rp);
                text(x-0.15, y+0.04, sprintf('(%.2f,%.2f)', refs(1, rp), refs(2, rp)));
            else
                box on;
                scatter3(pp_ref(1,:), pp_ref(2,:), pp_ref(3,:));
                x = refs(1, rp);
                y = refs(2, rp);   
                z = refs(3, rp);
                text(x+0.05, y-0.1, z, sprintf('(%.2f,%.2f, %.2f)', refs(1, rp), refs(2, rp), refs(3, rp)));
            end
            hold off;
        end


    end    
end