function mop = testmop( testname, dimension )
% Author: Asad Mohammadi
% Email : asadmoh1985@gmail.com
% --------
% License:
% --------
% This program is to be used under the terms of the GNU General Public License 
% (http://www.gnu.org/copyleft/gpl.html).
% Author: Asad Mohammadi
% Email : asadmoh1985@gmail.com
% Copyright notice: (c) 2014 Asad Mohammadi
% Get test multi-objective problems from a given name.
% The method get testing or benchmark problems for Multi-Objective
% Optimization. The implemented problems included ZDT, OKA, KNO.
% User get the corresponding test problem, which is an instance of class
% mop, by passing the problem name and optional dimension parameters.

mop=struct('name',[],'od',[],'pd',[],'domain',[],'func',[]);
switch lower(testname)
    case 'kno1'
        mop=kno1(mop);
    case 'zdt1'
        mop=zdt1(mop, dimension);
        case 'zdt2'
        mop=zdt2(mop, dimension);
        case 'zdt3'
        mop=zdt3(mop, dimension);
        case 'zdt4'
        mop=zdt4(mop,dimension );
        case 'zdt5'
        mop=zdt5(mop,dimension );        
        case 'zdt6'
        mop=zdt6(mop, dimension);
         case 'dtlz1_2d'
        mop=dtlz1_2d(mop, dimension);
         case 'dtlz1_3d'
        mop=dtlz1_3d(mop, dimension);
        case 'dtlz2'
        mop=dtlz2(mop, dimension);
    otherwise 
        error('Undefined test problem name');                
end 
end

%KNO1 function generator
function p=kno1(p)
 p.name='kno1';
 p.od = 2;
 p.pd = 2;
 p.domain= [0 3;0 3];
 p.func = @evaluate;
 
    %KNO1 evaluation function.
    function y = evaluate(x)
      y=zeros(2,1);
	  c = x(1)+x(2);
	  f = 9-(3*sin(2.5*c^0.5) + 3*sin(4*c) + 5 *sin(2*c+2));
	  g = (pi/2.0)*(x(1)-x(2)+3.0)/6.0;
	  y(1)= 20-(f*cos(g));
	  y(2)= 20-(f*sin(g)); 
    end
end

%ZDT1 function generator
function p=zdt1(p,dim)
 p.name='zdt1';
 p.pd=dim;
 p.od=2;
 p.domain=[zeros(dim,1) ones(dim,1)];
 p.func=@evaluate;
 
    %ZDT1 evaluation function.
    function y=evaluate(x)
        y=zeros(2,1);
        y(1) = x(1);
    	su = sum(x)-x(1);    
		g = 1 + 9 * su / (dim - 1);
		y(2) =g*(1 - sqrt(x(1) / g));
    end
end


%ZDT2 function generator
function p=zdt2(p, dim)
 p.name='zdt2';
 p.pd=dim;
 p.od=2;
 p.domain=[zeros(dim,1) ones(dim,1)];
 p.func=@evaluate;
 
    %ZDT2 evaluation function.
    function y=evaluate(x)
        y=zeros(2,1);
        y(1) = x(1);
    	su = sum(x)-x(1);    
		g = 1 + 9 * su / (dim - 1);
		y(2) =g*(1 -(x(1) / g)^2);
    end

end

%ZDT3 function generator
function p=zdt3(p,dim)
 p.name='zdt3';
 p.pd=dim;
 p.od=2;
 p.domain=[zeros(dim,1) ones(dim,1)];
 p.func=@evaluate;
 
    %ZDT3 evaluation function.
    function y=evaluate(x)
        y=zeros(2,1);
        y(1) = x(1);
    	su = sum(x)-x(1);    
		g = 1 + 9 * su / (dim - 1);
		y(2) = g * ( 1-sqrt(x(1)/g) - x(1)/g*sin(10*pi*x(1)));
    end

end


%ZDT4 function generator
function p=zdt4(p,dim)
 p.name='zdt4';
 p.pd=dim;
 p.od=2;
 lbounds = ones(dim,1)*-5;
 lbounds(1) = 0;
 ubounds = ones(dim,1)*5;
 ubounds(1) = 1;
 p.domain=[lbounds ubounds];
 p.func=@evaluate;
 
    %ZDT4 evaluation function.
    function y = evaluate(x)
       y=zeros(2,1);
       g = 1 + 10*(dim - 1);
       for i = 2:dim
            g = g + x(i)^2 - 10* cos(4*pi*x(i));
       end
       y(1) = x(1);
       y(2) = g * (1-sqrt(x(1)/g));
    end

end


function p=zdt5(p,dim)
 p.name='zdt5';
 p.pd=dim;
 p.od=2;
 p.domain=[zeros(dim,1) ones(dim,1) * 31];
 p.domain(1,2) = 2^30-1;
 p.func=@evaluate;
 
    %ZDT6 evaluation function.
    function y=evaluate(x)
        y=zeros(2,1);
        g = 0;
        for i=2:11
            g = g + v(u(x(i)));
        end

        y(1) = 1 + u(x(1));
        y(2)=g/y(1);
    end
end

function res = v(num)
    if (num < 5)
        res = 2 + num;
    elseif( num == 5)
        res = 1;
    end
end

function n = u(num)
    n = length(find(dec2bin(num) == '1'));
end

%ZDT6 function generator

function p=zdt6(p,dim)
 p.name='zdt6';
 p.pd=dim;
 p.od=2;
 p.domain=[zeros(dim,1) ones(dim,1)];
 p.func=@evaluate;
 
    %ZDT6 evaluation function.
    function y=evaluate(x)
        y=zeros(2,1);
        y(1) = 1 - exp(-4*x(1)) * sin(6*pi*x(1))^6;
    	g = 1 + 9 * (sum(x(2:end))/(dim-1))^0.25;
        y(2) = g * (1 - (y(1)/g)^2);
    end
end

%DTLZ1 function generator
function p=dtlz1_2d(p,dim)
 p.name='dtlz1_2d';
 p.pd=dim;
 p.od=2;
 p.domain=[zeros(dim,1) ones(dim,1)];
 p.func=@evaluate;
 
    %DTLZ1 evaluation function.
    function y=evaluate(x)
        y= zeros(p.od,1);    
        
        g = 100*(dim-2);
        temp = 0;
        for i=3:dim
            temp=temp+(x(i)-0.5)^2-cos(20*pi*(x(i)-0.5));
        end 
        g = g + 100 * temp;
        y(1) = (1+g)*x(1);
        y(2) = (1+g)*(1-x(1));
    end
end

%DTLZ1 function generator
function p=dtlz1_3d(p,dim)
 p.name='dtlz1_3d';
 p.pd=dim;
 p.od=3;
 p.domain=[zeros(dim,1) ones(dim,1)];
 p.func=@evaluate;
 
    %DTLZ1 evaluation function.
    function y=evaluate(x)
        y= zeros(p.od,1);    
        
        g = 100*(dim-2);
        temp = 0;
        for i=3:dim
            temp=temp+(x(i)-0.5)^2-cos(20*pi*(x(i)-0.5));
        end 
        g = g + 100 * temp;
        y(1) = (1+g)*x(1)*x(2);
        y(2) = (1+g)*x(1)*(1-x(2));
        y(3) = (1+g)*(1-x(1));
    end
end

%DTLZ2 function generator
function p=dtlz2(p,dim)
 p.name='dtlz2';
 p.pd=dim;
 p.od=3;
 p.domain=[zeros(dim,1) ones(dim,1)];
 p.func=@evaluate;
 
    %DTLZ2 evaluation function.
    function y=evaluate(x)
        y=zeros(p.od, 1);
        temp = x - 0.5;
        g = temp(3:end)' * temp(3:end);
        y(1) = (1+g) * cos(x(1)*pi/2) * cos(x(2)*pi/2);
        y(2) = (1+g) * cos(x(1)*pi/2) * sin(x(2)*pi/2);
        y(3) = (1+g) * sin(x(1)*pi/2);
    end

end


 